const webpackDevServer = require('webpack-dev-server');
const webpack = require('webpack');
const open = require('open');
const appConfig = require('./appConfig');
const getEnvironmentInfo = require('./extendEnv');
const { parseArgsToJson, getProjectPath } = require('./utils');

function openStartRun(webpackFactory) {
  process.env.BABEL_ENV = 'development';
  process.env.NODE_ENV = 'development';

  const PORT = process.env.PORT || parseArgsToJson.port || appConfig.startDefaultPort;
  const HOST = process.env.HOST || '0.0.0.0';

  const extendEnv = getEnvironmentInfo();

  const options = {
    contentBase: getProjectPath(appConfig.startOutput.path),
    hot: true,
    host: HOST,
    quiet: true,
    publicPath: appConfig.startOutput.publicPath || '/',
    compress: true,
    overlay: false,
    historyApiFallback: {
      // https://github.com/facebook/create-react-app/issues/387.
      disableDotRule: true
    },
    watchOptions: {
      aggregateTimeout: 300,
      poll: 1000,
      ignored: /node_modules/
    },
    disableHostCheck: true
  };

  const startOpenPath = appConfig.startOpenPath ? appConfig.startOpenPath : '';

  try {
    const config = webpackFactory({ ...appConfig, port: PORT, startOpenPath }, extendEnv);

    webpackDevServer.addDevServerEntrypoints(config, options);
    const compiler = webpack(config);
    // eslint-disable-next-line new-cap
    const server = new webpackDevServer(compiler, options);

    server.listen(PORT, HOST, err => {
      if (err) {
        // eslint-disable-next-line no-console
        console.log(err);
        return;
      }
      open(`http://localhost:${PORT}${startOpenPath}`);
    });

    ['SIGINT', 'SIGTERM'].forEach(sig => {
      process.on(sig, () => {
        server.close();
        process.exit();
      });
    });
  } catch (err) {
    if (err && err.message) {
      // eslint-disable-next-line no-console
      console.log(err.message);
    }
    process.exit(1);
  }
}

module.exports = openStartRun;
