const path = require('path');
const fs = require('fs');

// 执行node命令时候的文件绝对路径
const root = process.cwd();

/**
 * 从根目录开始获取文件绝对地址路径
 * @param filePath 文件参数
 * @returns {string} 返回绝对路径
 */
function getProjectPath(...filePath) {
  return path.join(root, ...filePath);
}

function getParseArgsToJson() {
  const args = process.argv.slice(2);
  const length = args.length - 1;
  const result = {};
  let count = 0;
  while (count <= length) {
    if (!/^--[0-9a-z]+$/.test(args[count])) {
      throw Error('Run args error, example: --port 3030');
    }
    const key = args[count].substring(2);
    count++;
    if (count > length) {
      result[key] = '';
      break;
    }
    result[key] = isNaN(Number(args[count])) ? args[count] : Number(args[count]);
    count++;
  }
  return result;
}

function isExistsSyncFile(path) {
  return fs.existsSync(path);
}

const parseArgsToJson = getParseArgsToJson();

function deleteFolders(path) {
  let files = [];
  if (isExistsSyncFile(path)) {
    files = fs.readdirSync(path);
    files.forEach(file => {
      const curPath = `${path}/${file}`;
      if (fs.statSync(curPath).isDirectory()) {
        deleteFolders(curPath);
      } else {
        fs.unlinkSync(curPath);
      }
    });
    fs.rmdirSync(path);
  }
}

module.exports = {
  getProjectPath,
  parseArgsToJson,
  deleteFolders,
  isExistsSyncFile
};
