const merge = require('webpack-merge');
const TerserPlugin = require('terser-webpack-plugin');
const ScriptExtHtmlWebpackPlugin = require('script-ext-html-webpack-plugin');
// eslint-disable-next-line import/no-extraneous-dependencies
const OptimizeCSSAssetsPlugin = require('optimize-css-assets-webpack-plugin');
const safePostCssParser = require('postcss-safe-parser');
const baseConfig = require('./webpack.base');
const { getProjectPath, deleteFolders } = require('../common/utils');

module.exports = function(appConfig, extendEnv) {
  const outputPath = getProjectPath(appConfig.buildOutput.path);
  deleteFolders(outputPath);

  return merge(baseConfig(true, appConfig, extendEnv), {
    // 打包的入口文件
    entry: getProjectPath(appConfig.buildEntry),
    // 打包的出口
    output: {
      publicPath: appConfig.buildOutput.publicPath || '/',
      path: outputPath,
      // 入口文件生产的js
      filename: `js/[name].chunk.js?v=${extendEnv.version}`,
      // 非入口文件生产的js
      chunkFilename: `js/[name].chunk.js?v=${extendEnv.version}`
    },
    node: {
      fs: 'empty'
    },
    plugins: [
      new ScriptExtHtmlWebpackPlugin({
        defaultAttribute: 'async',
        preload: /\.js\?v=/
      })
    ],
    optimization: {
      concatenateModules: true,
      minimizer: [
        // 压缩js
        new TerserPlugin({
          cache: true,
          parallel: true,
          sourceMap: true
        }),

        // 压缩css
        new OptimizeCSSAssetsPlugin({
          cssProcessorOptions: {
            parser: safePostCssParser,
            map: {
              inline: false,
              annotation: false
            }
          }
        })
      ],
      splitChunks: {
        chunks: 'initial',
        minSize: 0,
        maxAsyncRequests: 5,
        maxInitialRequests: 3,
        automaticNameDelimiter: '~',
        name: true,
        cacheGroups: {
          vendor: {
            test: /[\\/]node_modules[\\/]/,
            chunks: 'initial',
            name: 'vendor',
            minChunks: 1,
            priority: 10
          }
        }
      },
      runtimeChunk: {
        name: 'manifest'
      }
    }
  });
};
