const webpack = require('webpack');
const merge = require('webpack-merge');
const path = require('path');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const webpackCommon = require('../common/webpack.common');

module.exports = (appConfig, extendEnv, { isServer, isHotServer, isSetCommonAlias, isProd }) => {
  const result = {
    plugins: [],
    module: {
      rules: []
    }
  };
  if ((isProd || isHotServer) && !isServer) {
    result.plugins.push(
      // modules Id 随机数导致缓存失效
      new webpack.HashedModuleIdsPlugin(),
      // 解决新旧版本迭代时，google cache 旧版不能正常运行js
      // 通过指定chunk name而不是随机数
      new webpack.NamedChunksPlugin(chunk => {
        if (chunk.name) {
          return chunk.name;
        }

        // eslint-disable-next-line no-underscore-dangle
        return [...chunk._modules]
          .map(m =>
            path.relative(m.context, m.userRequest.substring(0, m.userRequest.lastIndexOf('.')))
          )
          .join('_');
      }),
      new MiniCssExtractPlugin({
        // filename: isHotServer ? 'css/[name].css' : `css/[name].css?v=${env.version}`,
        // 采用AMP 直接 把css 填充到html
        filename: isHotServer ? 'css/[name].css' : 'css/[name][contenthash].css',
        allChunks: !isHotServer,
        ignoreOrder: true
      })
    );
  }

  return merge(
    webpackCommon(appConfig, extendEnv, { isProd, isServer, isHotServer, isSetCommonAlias }),
    result
  );
};
