const webpack = require('webpack');
// eslint-disable-next-line import/no-extraneous-dependencies
const nodeExternals = require('webpack-node-externals');
const merge = require('webpack-merge');
const baseConfig = require('./webpack.base');
const { getProjectPath } = require('../common/utils');

module.exports = function(appConfig, extendEnv) {
  return merge(
    baseConfig(appConfig, extendEnv, {
      isServer: true,
      isHotServer: true,
      isSetCommonAlias: true,
      isProd: false
    }),
    {
      // 告诉 webpack打包的是 node端代码，避免把nodejs内置的模块打包到输出文件中，例如 fs path
      target: 'node',
      // 避免把 node_modules包下的第三方模块中包含的 nodejs内置的模块打包到出书文件中
      externals: nodeExternals({
        // 当然外部的css还是可以打进来的
        whitelist: /\.(js|css|less|scss|png|jpg|jpeg)$/
      }),
      // 入口文件
      entry: getProjectPath(appConfig.hotSsrServerEntry),
      // 输出信息
      output: {
        path: getProjectPath(appConfig.hotSsrServerOutput.path),
        publicPath: appConfig.hotSsrServerOutput.publicPath || '/',
        // 输出文件名
        filename: 'bundle.js',
        libraryTarget: 'commonjs2'
      },
      module: {
        rules: [
          {
            oneOf: [
              {
                test: /\.(scss|css|sass)$/,
                use: [
                  {
                    loader: 'ignore-loader'
                  }
                ]
              }
            ]
          }
        ]
      },
      plugins: [new webpack.ContextReplacementPlugin(/any-promise/)],
      // 避免路径丢失
      node: {
        __filename: true,
        __dirname: true
      }
    }
  );
};
