const merge = require('webpack-merge');
const LoadablePlugin = require('@loadable/webpack-plugin');
const TerserPlugin = require('terser-webpack-plugin');
// eslint-disable-next-line import/no-extraneous-dependencies
const OptimizeCSSAssetsPlugin = require('optimize-css-assets-webpack-plugin');
const safePostCssParser = require('postcss-safe-parser');
const baseConfig = require('./webpack.base');

const { getProjectPath } = require('../common/utils');

const isCommonBuild = process.env.REACT_APP_BUILD_TYPE === 'common';

module.exports = function(appConfig, extendEnv) {
  return merge(
    baseConfig(appConfig, extendEnv, {
      isServer: false,
      isHotServer: false,
      isSetCommonAlias: isCommonBuild,
      isProd: true
    }),
    {
      entry: getProjectPath(appConfig.buildClientEntry),
      // 打包的出口
      output: {
        path: getProjectPath(appConfig.buildClientOutput.path),
        publicPath: appConfig.buildClientOutput.publicPath || '/',
        // 入口文件生产的js
        filename: `js/[name].js?v=${extendEnv.version}`,
        // 非入口文件生产的js
        chunkFilename: `js/[name].js?v=${extendEnv.version}`
      },
      node: {
        fs: 'empty'
      },
      plugins: [
        new LoadablePlugin({
          filename: appConfig.buildClientLoadableConfig.filename,
          writeToDisk: {
            filename: getProjectPath(appConfig.buildClientLoadableConfig.writeToDiskFilename)
          }
        })
      ],
      optimization: {
        minimizer: [
          // 压缩js
          new TerserPlugin({
            cache: true,
            parallel: true,
            sourceMap: true
          }),

          // 压缩css
          new OptimizeCSSAssetsPlugin({
            cssProcessorOptions: {
              parser: safePostCssParser,
              map: {
                inline: false,
                annotation: false
              }
            }
          })
        ],
        splitChunks: {
          chunks: 'initial',
          minSize: 0,
          maxAsyncRequests: 5,
          maxInitialRequests: 3,
          automaticNameDelimiter: '~',
          name: true,
          cacheGroups: {
            common: {
              chunks: 'async',
              name: 'common',
              minChunks: 20,
              priority: 2
            },
            vendor: {
              test: /[\\/]node_modules[\\/]/,
              chunks: 'initial',
              name: 'vendor',
              minChunks: 1,
              priority: 10
            }
          }
        },
        runtimeChunk: {
          name: 'manifest'
        }
      }
    }
  );
};
