// eslint-disable-next-line import/no-extraneous-dependencies
const nodeExternals = require('webpack-node-externals');
const merge = require('webpack-merge');
const baseConfig = require('./webpack.base');
const { getProjectPath } = require('../common/utils');

const isCommonBuild = process.env.REACT_APP_BUILD_TYPE === 'common';

module.exports = function(appConfig, extendEnv) {
  return merge(
    baseConfig(appConfig, extendEnv, {
      isServer: true,
      isHotServer: false,
      isSetCommonAlias: isCommonBuild,
      isProd: true
    }),
    {
      target: 'node',
      entry: getProjectPath(appConfig.buildServerEntry),
      output: {
        path: getProjectPath(appConfig.buildServerOutput.path),
        publicPath: appConfig.buildServerOutput.publicPath || '/',
        filename: 'server-bundle.js',
        libraryTarget: 'commonjs2',
        chunkFilename: 'js/[name].[chunkhash].js'
      },
      module: {
        rules: [
          {
            oneOf: [
              {
                test: /\.(scss|css|sass)$/,
                use: [
                  {
                    loader: 'ignore-loader'
                  }
                ]
              }
            ]
          }
        ]
      },
      // 服务端打包的时候忽略外部的npm包
      externals: nodeExternals({
        // 当然外部的css还是可以打进来的
        whitelist: /\.(css|less|scss)$/
      })
    }
  );
};
